namespace MessageAPI.RealTimePresence
{

    public class RealTimeMessageHub : Hub
    {
        private readonly IMessageDataRepository MessageDataRepository;
        private readonly IMapper Mapper;
        private readonly IUserDataRepository UserDataRepository;
        public RealTimeMessageHub(IMessageDataRepository messageDataRepository, IMapper mapper, IUserDataRepository userDataRepository)
        {
            MessageDataRepository = messageDataRepository;
            Mapper = mapper;
            UserDataRepository = userDataRepository;
        }
       

        public override async Task OnConnectedAsync()
        {
            var http = Context.GetHttpContext();
            var userInChat = http.Request.Query["APIUser"].ToString(); 
            var chatName = ChatName(Context.User.FindFirst(ClaimTypes.NameIdentifier)?.Value, userInChat);
            await Groups.AddToGroupAsync(Context.ConnectionId, chatName);
            await NewSignalRGroup(Context, chatName);
            var texts = await MessageDataRepository.MessageBetweenUsers(Context.User.FindFirst(ClaimTypes.NameIdentifier)?.Value, userInChat);
            await Clients.Group(chatName).SendAsync("GetMessage", texts);
        }

        public override async Task OnDisconnectedAsync(Exception exception)
        {
            await DeleteSignalRGroup(Context.ConnectionId);
            await base.OnDisconnectedAsync(exception); //we dont need to do here anything becuase SignalR will remove disconnected user itself;
        }

        public async Task MessageWithUser(NewMessageViewItem newMessageViewItem)
        {
            var name = Context.User.FindFirst(ClaimTypes.NameIdentifier)?.Value;
            if(name == newMessageViewItem.TargetUsername) throw new HubException("Cant message to yourself");
            var issuer = await UserDataRepository.GetByUsername(name);
            var target  = await UserDataRepository.GetByUsername(newMessageViewItem.TargetUsername);
            if(target == null) throw new HubException("Target user couldn't found");
            var newMessage = new Message
            {
                Issuer = issuer,
                Target = target,
                IssuerUsername = issuer.UserName,
                TargetUsername = target.UserName,
                Text = newMessageViewItem.Text
            };
            var chatName = ChatName(issuer.UserName, target.UserName);
            var chat = await MessageDataRepository.AddNewSignalRGroup(chatName);
            if(chat.UsersSignalR.Any(p => p.Name == target.UserName))
            {
                newMessage.TextRead = DateTime.UtcNow;
            }
            MessageDataRepository.newMessage(newMessage);
            if(await MessageDataRepository.Save()){
                
                await Clients.Group(chatName).SendAsync("NewChat", Mapper.Map<MessageViewItem>(newMessage));
            } 
        }

        private async Task<bool> NewSignalRGroup(HubCallerContext hubCallerContext, string name )
        {
            var newSignalRG = await MessageDataRepository.AddNewSignalRGroup(name);
            var userSignalR = new UserSignalR(hubCallerContext.ConnectionId, hubCallerContext.User.FindFirst(ClaimTypes.NameIdentifier)?.Value);
            if(newSignalRG == null)
            {
                newSignalRG = new SignalRGroup(name);
                MessageDataRepository.NewSignalRGroup(newSignalRG);
            }
            newSignalRG.UsersSignalR.Add(userSignalR);
            return await MessageDataRepository.Save();
        }

        private async Task DeleteSignalRGroup(string id)
        {
            var userSignalR = await MessageDataRepository.NewUsersSignalR(id);
            MessageDataRepository.RemoveUsersSignalR(userSignalR);
            await MessageDataRepository.Save();
        }

        private string ChatName(string sender, string otherSender )
        {
            var ordering = string.CompareOrdinal(sender, otherSender) < 0;
            return ordering ? $"{sender}-{otherSender}" : $"{otherSender}-{sender}";
        }
    }
}